@@ -1,8 +1,8 @@
-/*  $Id: AfternoonStalkerEngine.cpp,v 1.28 2004/05/02 04:55:45 sarrazip Exp $
+/*  $Id: AfternoonStalkerEngine.cpp,v 1.42 2009/02/27 03:08:33 sarrazip Exp $
     AfternoonStalkerEngine.cpp - A robot-killing video game engine.
 
     afternoonstalker - A robot-killing video game.
-    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2009 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -40,6 +40,7 @@
 #include <vector>
 #include <string>
 #include <algorithm>
+#include <iostream>
 #include <iomanip>
 
 using namespace std;
@@ -65,8 +66,6 @@ using namespace std;
 #include "images/wall_end_tile_1.xpm"
 #include "images/wall_end_tile_2.xpm"
 #include "images/wall_end_tile_3.xpm"
-#include "images/bomb_explosion_0.xpm"
-#include "images/bomb_explosion_1.xpm"
 
 
 /*  Sprite images:
@@ -107,7 +106,6 @@ using namespace std;
 #include "images/invisible_robot_explosion_1.xpm"
 
 #include "images/gun.xpm"
-#include "images/bomb.xpm"
 #include "images/spider_right_0.xpm"
 #include "images/spider_right_1.xpm"
 #include "images/spider_up_0.xpm"
@@ -156,11 +154,9 @@ static const int
     NUM_INIT_LIVES = 5,
     TILE_MATRIX_Y_OFFSET = TILE_SIDE,
     PLAYER_SPEED_FACTOR = 4,
-    GRAY_ROBOT_SPEED_FACTOR = 3,
     SPIDER_SPEED_FACTOR = 2,
     BAT_SPEED_FACTOR = 3,
     BULLET_SPEED_FACTOR = 8,
-    BULLETS_PER_GUN = 6,
     SPIDER_SCORE = 100,
     BAT_SCORE = 300;
 
@@ -262,11 +258,9 @@ countDirections(const bool d[4])
 
 AfternoonStalkerEngine::AfternoonStalkerEngine(
 			    const string &windowManagerCaption,
-			    bool useSound,
-			    bool pedantic,
 			    bool fullScreen,
-			    bool onlyBlueRobots,
-			    const string &setFilename)
+			    const std::string &_mazeFilename,
+			    Rules _rules)
 				throw(int, string)
 
   : GameEngine(Couple(640, 416), windowManagerCaption, fullScreen),
@@ -274,6 +268,10 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
 
     paused(false),
     tickCount(0),
+    useSound(true),
+    soundChunksInitialized(false),
+    rules(_rules),
+    mazeFilename(_mazeFilename),
 
     blackColor(0),
 
@@ -281,9 +279,6 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
     tileMatrix(),
 
     theScore(0),
-    scorePos(3, 6),
-
-    playerLivesPos(theScreenSizeInPixels.x - 10 * 7 - 3, 6),
 
     playerPA(4),
     playerSprite(NULL),
@@ -293,7 +288,6 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
     lastPlayerDir(-1),
     lastRequestedDir(-1),
     playerParalysisTime(0),
-    playerHasBomb(false),
 
     playerBulletPA(1),
     playerBulletSprites(),
@@ -304,15 +298,6 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
     gunPA(1),
     gunSprites(),
 
-    bombPA(1),
-    bombSprites(),
-    timeUntilNextBomb(0),
-
-    theBomb(),
-    bombExplosionTicks(0),
-    bombExplosionPA(2),
-    bombExplosionSprites(),
-
     grayRobotPA(2),
     blueRobotPA(2),
     whiteRobotPA(2),
@@ -321,10 +306,8 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
     blinkPA(1),
     robotSprites(),
     timeUntilNextRobot(0),
-    sendOnlyBlueRobots(onlyBlueRobots),
     initRobotPos(),
     numCreatedRobots(0),
-    maxNumRobots(0),
 
     robotBulletPA(1),
     bigRobotBulletPA(3),
@@ -352,24 +335,12 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
     digitPA(10),
     scoreSprites(),
 
-    quitKS(SDLK_ESCAPE),
-    leftKS(SDLK_LEFT),
-    rightKS(SDLK_RIGHT),
-    upKS(SDLK_UP),
-    downKS(SDLK_DOWN),
-    startKS(SDLK_SPACE),
-    shootKS(SDLK_LCTRL, SDLK_RCTRL),
-    pauseKS(SDLK_p),
-    bombKS(SDLK_b),
-    shootLeftKS(SDLK_h),
-    shootRightKS(SDLK_l),
-    shootUpKS(SDLK_k),
-    shootDownKS(SDLK_j),
+    controller(),
+
+    currentMenu(&mainMenu),
 
     theSoundMixer(NULL)
 {
-    this->useSound = useSound;
-
     blackColor = SDL_MapRGB(theSDLScreen->format, 0, 0, 0);
 
     try
@@ -384,7 +355,7 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
 	    throw msg;
 	}
 
-	doOneTimeInitializations(useSound, pedantic, setFilename);
+	doOneTimeInitializations();
     }
     catch (string &msg)
     {
@@ -417,7 +388,6 @@ AfternoonStalkerEngine::initializeLevel()
 
     numCreatedRobots = (initNumCreatedRobots != NULL
     			? atoi(initNumCreatedRobots) : 0);
-    maxNumRobots = 3;
 
     initializePlayerLife();
 }
@@ -427,6 +397,7 @@ void
 AfternoonStalkerEngine::initializePlayerLife()
 {
     playerSprite->setPos(initPlayerPos);
+    playerSprite->currentPixmapIndex = 0;
 
     numPlayerBullets = 0;
     lastPlayerDir = -1;
@@ -436,7 +407,6 @@ AfternoonStalkerEngine::initializePlayerLife()
     deleteSprites(playerBulletSprites);
     deleteSprites(playerExplosionSprites);
     deleteSprites(gunSprites);
-    deleteSprites(bombSprites);
     deleteSprites(robotSprites);
     deleteSprites(robotBulletSprites);
     deleteSprites(robotExplosionSprites);
@@ -444,24 +414,9 @@ AfternoonStalkerEngine::initializePlayerLife()
     deleteSprites(spiderExplosionSprites);
     deleteSprites(batSprites);
     deleteSprites(batExplosionSprites);
-    deleteSprites(bombExplosionSprites);
 
-    setTimeUntilNextBomb();
     setTimeUntilNextRobot();
     setTimeUntilNextAnimal();
-
-    theBomb.reset();
-    bombExplosionTicks = 0;
-}
-
-
-void
-AfternoonStalkerEngine::setTimeUntilNextBomb()
-{
-    if (theScore >= 30000)
-	timeUntilNextBomb = Rnd(FPS * 2, FPS * 5);
-    else
-	timeUntilNextBomb = Rnd(FPS * 45, FPS * 120);
 }
 
 
@@ -498,89 +453,231 @@ getDir(const char *defaultValue, const char *envVarName)
 
 
 void
-AfternoonStalkerEngine::doOneTimeInitializations(
-				bool useSound,
-				bool pedantic,
-				const string &setFilename)
-					throw(string)
+AfternoonStalkerEngine::doOneTimeInitializations() throw(string)
 /*  Throws an error message in a 'string' if an error occurs.
 */
 {
-    string pkgdatadir = getDir(PKGDATADIR, "PKGDATADIR");
-    string pkgsounddir = getDir(PKGSOUNDDIR, "PKGSOUNDDIR");
-
-    loadLevel(1, pkgdatadir + setFilename);
-    assert(initPlayerPos.x >= 0);
-    assert(initPlayerPos.y >= 0);
+    loadLevel();
 
+    assert(playerSprite == NULL);
     playerSprite = new Sprite(playerPA, initPlayerPos,
 				Couple(0, 0), Couple(0, 0), Couple(4, 4),
 				playerPA.getImageSize() - Couple(8, 8));
 
 
-    /*  Initialize sound system and load sound files, if desired.
-	If useSound is false, then theSoundMixer remains NULL and
-	the SoundMixer::Chunk objects remain uninitialized.
-    */
+    initMenus();
+}
+
+
+void
+AfternoonStalkerEngine::initSounds()
+{
     if (useSound)
     {
 	try
 	{
-	    theSoundMixer = NULL;
-	    theSoundMixer = new SoundMixer();
+	    if (theSoundMixer == NULL)
+		theSoundMixer = new SoundMixer();
 	}
 	catch (const SoundMixer::Error &e)
 	{
+	    assert(theSoundMixer == NULL);
 	    return;
 	}
 
-	try
+	if (!soundChunksInitialized)
 	{
-	    gunPickupSound.init(pkgsounddir + "gun-pickup.wav");
-	    playerBulletSound.init(pkgsounddir + "player-bullet.wav");
-	    shootingBlanksSound.init(pkgsounddir + "shooting-blanks.wav");
-	    playerHitSound.init(pkgsounddir + "player-hit.wav");
-	    robotBulletSound.init(pkgsounddir + "robot-bullet.wav");
-	    robotHitSound.init(pkgsounddir + "robot-hit.wav");
-	    batKilledSound.init(pkgsounddir + "bat-killed.wav");
-	    spiderKilledSound.init(pkgsounddir + "spider-killed.wav");
-	    newLifeSound.init(pkgsounddir + "new-life.wav");
-	    bombExplodesSound.init(pkgsounddir + "bomb-explodes.wav");
-	}
-	catch (const SoundMixer::Error &e)
-	{
-	    throw e.what();
-	}
+	    try
+	    {
+		string pkgsounddir = getDir(PKGSOUNDDIR, "PKGSOUNDDIR");
 
-	try
-	{
-	    if (pedantic)
-		missedSound.init(pkgsounddir + "missed.wav");
-	}
-	catch (const SoundMixer::Error &e)
-	{
-	    // Don't use this (lame) sound if it is mysteriously absent...
+		gunPickupSound.init(pkgsounddir + "gun-pickup.wav");
+		playerBulletSound.init(pkgsounddir + "player-bullet.wav");
+		shootingBlanksSound.init(pkgsounddir + "shooting-blanks.wav");
+		playerHitSound.init(pkgsounddir + "player-hit.wav");
+		robotBulletSound.init(pkgsounddir + "robot-bullet.wav");
+		robotHitSound.init(pkgsounddir + "robot-hit.wav");
+		batKilledSound.init(pkgsounddir + "bat-killed.wav");
+		spiderKilledSound.init(pkgsounddir + "spider-killed.wav");
+		newLifeSound.init(pkgsounddir + "new-life.wav");
+	    }
+	    catch (const SoundMixer::Error &e)
+	    {
+		// Not supposed to happen if package correctly installed.
+		// Print trace for debugging purposes.
+		cerr << e.what() << endl;
+	    }
+	    
+	    soundChunksInitialized = true;
 	}
     }
+    else
+    {
+	delete theSoundMixer;
+	theSoundMixer = NULL;
+    }
 }
 
 
 void
-AfternoonStalkerEngine::loadLevel(int levelNo, const string &setFilename)
-								throw(string)
-/*  Loads the level described in the file designated by 'setFilename'
-    and associates it with the given level number.
+AfternoonStalkerEngine::initMenus()
+{
+    mainMenu.setTitle("Move cursor with arrow keys - select with Space bar:");
+    mainMenu.addEntry("Start Game");
+    mainMenu.addEntry("Options");
+    mainMenu.addEntry("Instructions");
+    mainMenu.addEntry("Quit");
+    mainMenu.setParentMenu(&mainMenu);
 
-    Replaces the contents of the members 'tileMatrix' and 'initPlayerPos'.
+    optionsMenu.setTitle("Options:");
+    optionsMenu.addEntry("Rules");
+    optionsMenu.addEntry("Maze");
+    optionsMenu.addEntry("Sounds");
+    optionsMenu.addEntry("Back");
+    optionsMenu.setParentMenu(&mainMenu);
+
+    rulesMenu.setTitle("Rules of the game:");
+    rulesMenu.addEntry("Original (2001)");
+    rulesMenu.addEntry("Alternative (2008)");
+    rulesMenu.setParentMenu(&optionsMenu);
+    rulesMenu.setSelectionIndex(rules == AS_RULES_1 ? 0 : 1);
+
+    mazeMenu.setTitle("Maze:");
+    mazeMenu.addEntry("Night Stalker (original)");
+    mazeMenu.addEntry("Alternative");
+    mazeMenu.setParentMenu(&optionsMenu);
+    mazeMenu.setSelectionIndex(mazeFilename == "level1.set" ? 0 : 1);
+
+    soundsMenu.setTitle("Use sound effects?");
+    soundsMenu.addEntry("Yes");
+    soundsMenu.addEntry("No");
+    soundsMenu.setParentMenu(&optionsMenu);
+
+    instructionsMenu.setTitle(
+	"Move with arrow keys or joystick. Shoot with Space, h-j-k-l, or A-B-X-Y.");
+    instructionsMenu.addEntry("Back");
+    instructionsMenu.setParentMenu(&mainMenu);
+
+    pauseMenu.setTitle("Game paused. Select with keyboard arrows and Space bar:");
+    pauseMenu.addEntry("Resume Current Game");
+    pauseMenu.addEntry("End Current Game");
+    pauseMenu.addEntry("Quit Program");
+    pauseMenu.setParentMenu(&pauseMenu);
+}
+
+
+AfternoonStalkerEngine::MenuAction
+AfternoonStalkerEngine::processMenuMode()
+{
+    assert(currentMenu != NULL);
+
+    displayMessage(2, currentMenu->getTitle().c_str());
+    string entriesLine;
+    for (size_t i = 0; i < currentMenu->getNumEntries(); ++i)
+    {
+	bool entrySelected = (currentMenu->getSelectionIndex() == i);
+	string entry = (entrySelected ? "->" : "  ")
+			+ currentMenu->getEntry(i)
+			+ (entrySelected ? "<-" : "  ");
+	if (i > 0)
+	    entriesLine += "   ";
+	entriesLine += entry;
+    }
+    displayMessage(3, entriesLine.c_str());
+
+    if (controller.isNextMenuEntryRequested())
+	currentMenu->nextSelection();
+    else if (controller.isPrevMenuEntryRequested())
+	currentMenu->prevSelection();
+    else if (controller.isMenuReturnRequested())
+    {
+	currentMenu = currentMenu->getParentMenu();
+	assert(currentMenu != NULL);
+    }
+    else if (controller.isMenuSelectionRequested())
+    {
+	// Interpret menu selection.
+
+	if (currentMenu == &mainMenu)
+	{
+	    switch (currentMenu->getSelectionIndex())
+	    {
+		case 0:  return START_GAME;
+		case 1:  currentMenu = &optionsMenu; break;
+		case 2:  currentMenu = &instructionsMenu; break;
+		case 3:  return QUIT_PROGRAM;
+	    }
+	}
+	else if (currentMenu == &instructionsMenu)
+	{
+	    currentMenu = &mainMenu;
+	}
+	else if (currentMenu == &optionsMenu)
+	{
+	    switch (currentMenu->getSelectionIndex())
+	    {
+		case 0:  currentMenu = &rulesMenu; break;
+		case 1:  currentMenu = &mazeMenu; break;
+		case 2:  currentMenu = &soundsMenu; break;
+		case 3:  currentMenu = &mainMenu; break;
+	    }
+	}
+	else if (currentMenu == &rulesMenu)
+	{
+	    switch (currentMenu->getSelectionIndex())
+	    {
+		case 0: rules = AS_RULES_1; break;
+		case 1: rules = AS_RULES_2; break;
+	    }
+	    currentMenu = &optionsMenu;
+	}
+	else if (currentMenu == &mazeMenu)
+	{
+	    switch (currentMenu->getSelectionIndex())
+	    {
+		case 0: mazeFilename = "level1.set"; break;
+		case 1: mazeFilename = "level2.set"; break;
+	    }
+	    currentMenu = &optionsMenu;
+	}
+	else if (currentMenu == &soundsMenu)
+	{
+	    useSound = (currentMenu->getSelectionIndex() == 0);
+	    currentMenu = &optionsMenu;
+	}
+	else if (currentMenu == &pauseMenu)
+	{
+	    switch (currentMenu->getSelectionIndex())
+	    {
+		case 0: paused = false; break;
+		case 1: return QUIT_GAME;
+		case 2: return QUIT_PROGRAM;
+	    }
+	}
+    }
+
+    return STAY_IN_MENUS;
+}
+
+
+void
+AfternoonStalkerEngine::loadLevel() throw(string)
+/*  Loads the level described in the file designated by 'mazeFilename'.
+
+    Replaces the contents of the members tileMatrix, initPlayerPos,
+    initRobotPos and initSpiderPos.
 
     The next-to-last row of a level must have at least one floor tile.
 
     Throws an error message in a 'string' if an error occurs.
 */
 {
-    ifstream setFile(setFilename.c_str());
+    string pkgdatadir = getDir(PKGDATADIR, "PKGDATADIR");
+    string fullFilename = pkgdatadir + mazeFilename;
+
+    ifstream setFile(fullFilename.c_str());
     if (!setFile)
-	throw string("Could not open " + setFilename);
+	throw string("Could not open " + fullFilename);
     
     size_t levelWidth = 0, levelHeight = 0;
     string line;
@@ -717,6 +814,10 @@ AfternoonStalkerEngine::loadLevel(int levelNo, const string &setFilename)
 
     // Center sprite rectangle in tile:
     initSpiderPos -= spiderPA.getImageSize() / 2;
+
+
+    assert(initPlayerPos.x >= 0);
+    assert(initPlayerPos.y >= 0);
 }
 
 
@@ -727,19 +828,7 @@ AfternoonStalkerEngine::loadLevel(int levelNo, const string &setFilename)
 void
 AfternoonStalkerEngine::processKey(SDLKey keysym, bool pressed)
 {
-    quitKS.check(keysym, pressed);
-    leftKS.check(keysym, pressed);
-    rightKS.check(keysym, pressed);
-    upKS.check(keysym, pressed);
-    downKS.check(keysym, pressed);
-    startKS.check(keysym, pressed);
-    shootKS.check(keysym, pressed);
-    pauseKS.check(keysym, pressed);
-    bombKS.check(keysym, pressed);
-    shootLeftKS.check(keysym, pressed);
-    shootDownKS.check(keysym, pressed);
-    shootUpKS.check(keysym, pressed);
-    shootRightKS.check(keysym, pressed);
+    controller.processKey(keysym, pressed);
 }
 
 
@@ -747,8 +836,12 @@ AfternoonStalkerEngine::processKey(SDLKey keysym, bool pressed)
 bool
 AfternoonStalkerEngine::tick()
 {
-    if (quitKS.isPressed())
-	return false;
+    if (numPlayerLives > 0 && controller.isMenuRequested())
+    {
+	paused = true;
+	currentMenu = &pauseMenu;
+	currentMenu->setSelectionIndex(0);  // make "resume" default selection
+    }
 
     if (paused)
     {
@@ -757,12 +850,11 @@ AfternoonStalkerEngine::tick()
 	restoreForeground();
 	drawScoreBoard();
 
-	if (pauseKS.justPressed())
-	{
-	    paused = false;
-	}
-	else
-	    displayPauseMessage();
+	MenuAction action = processMenuMode();  // allow user to select in current menu
+	if (action == QUIT_PROGRAM)
+	    return false;
+	if (action == QUIT_GAME)
+	    endCurrentGame();
     }
     else
     {
@@ -776,17 +868,25 @@ AfternoonStalkerEngine::tick()
 	drawScoreBoard();
 
 
-	if (numPlayerLives == 0)
+	if (numPlayerLives == 0)  // if demo mode
 	{
-	    if (startKS.justPressed())
+	    MenuAction action = processMenuMode();  // allow user to select in current menu
+	    if (action == QUIT_PROGRAM)
+		return false;
+
+	    if (action == START_GAME)
 	    {
+		loadLevel();
+
+		initSounds();  // apply 'useSound' as possibly set by user in Options menu
+
 		theScore = getenv("INITSCORE") ? atol(getenv("INITSCORE")) : 0;
 		initializeLevel();
-		addToNumPlayerLives(5);
+		numPlayerLives += NUM_INIT_LIVES;
 	    }
-	    else
-		displayStartMessage();
 	}
+	else
+	    displayMessage(3, "Press Escape key to pause or quit game");
 	
 
 	if (numPlayerLives > 0)
@@ -822,14 +922,9 @@ AfternoonStalkerEngine::tick()
 	}
     }
 
-    startKS.remember();
-    shootKS.remember();
-    pauseKS.remember();
-    bombKS.remember();
-    shootLeftKS.remember();
-    shootRightKS.remember();
-    shootUpKS.remember();
-    shootDownKS.remember();
+    displayMessage(0, "Afternoon Stalker " VERSION " - by Pierre Sarrazin");
+
+    controller.update();
 
     return true;
 }
@@ -840,14 +935,6 @@ AfternoonStalkerEngine::animatePlayer()
 /*  Returns true if the game must continue, or false to have it stop.
 */
 {
-    if (pauseKS.justPressed())
-    {
-	paused = true;
-	displayPauseMessage();
-	return true;
-    }
-
-
     if (!playerExplosionSprites.empty())  // if player is dying
 	return true;
 
@@ -867,41 +954,35 @@ AfternoonStalkerEngine::animatePlayer()
     /*  Register the last requested direction, which will be the direction
 	of the player's bullet, when applicable.
     */
-    if (rightKS.isPressed())
+    if (controller.isRightRequested())
 	lastRequestedDir = RIGHT;
-    else if (upKS.isPressed())
+    else if (controller.isUpRequested())
 	lastRequestedDir = UP;
-    else if (leftKS.isPressed())
+    else if (controller.isLeftRequested())
 	lastRequestedDir = LEFT;
-    else if (downKS.isPressed())
+    else if (controller.isDownRequested())
 	lastRequestedDir = DOWN;
 
-    if (shootKS.justPressed())
+    if (controller.isShootRequested())
 	makePlayerShoot(lastRequestedDir == -1 ? UP : lastRequestedDir);
-    else if (shootLeftKS.justPressed())
+    else if (controller.isShootLeftRequested())
 	makePlayerShoot(LEFT);
-    else if (shootRightKS.justPressed())
+    else if (controller.isShootRightRequested())
 	makePlayerShoot(RIGHT);
-    else if (shootUpKS.justPressed())
+    else if (controller.isShootUpRequested())
 	makePlayerShoot(UP);
-    else if (shootDownKS.justPressed())
+    else if (controller.isShootDownRequested())
 	makePlayerShoot(DOWN);
 
-    if (bombKS.justPressed())
-    {
-	if (playerHasBomb)
-	    layPlayerBomb();
-	else
-	    playSoundEffect(shootingBlanksSound);
-    }
-
     playerSprite->currentPixmapIndex = 0;
 	    // use "stopped" image unless a non-zero speed is chosen
 
     bool desiredDirs[4] =
 	{
-	    rightKS.isPressed(), upKS.isPressed(),
-	    leftKS.isPressed(), downKS.isPressed(),
+	    controller.isRightRequested(),
+	    controller.isUpRequested(),
+	    controller.isLeftRequested(),
+	    controller.isDownRequested()
 	};
     playerSpeed = attemptMove(*playerSprite, desiredDirs, PLAYER_SPEED_FACTOR);
     if (playerSpeed.isZero())
@@ -976,14 +1057,6 @@ AfternoonStalkerEngine::makePlayerShoot(int shotDir)
 }
 
 
-void
-AfternoonStalkerEngine::layPlayerBomb()
-{
-    theBomb.init(playerSprite->getCenterPos(), 3 * 20, 32 * 4);
-    playerHasBomb = false;
-}
-
-
 Couple
 AfternoonStalkerEngine::attemptMove(const Sprite &s,
 				    const bool attemptedDirections[4],
@@ -1331,13 +1404,6 @@ AfternoonStalkerEngine::animateAutomaticCharacters()
     moveExplosionSprites(batExplosionSprites);
 
 
-    // Animate the bomb explosion, if applicable:
-    if ((tickCount & 0x01) == 0)
-	for (SpriteList::iterator it = bombExplosionSprites.begin();
-				it != bombExplosionSprites.end(); it++)
-	    (**it).currentPixmapIndex ^= 1;
-
-
     bool playerDying = !playerExplosionSprites.empty();
     moveExplosionSprites(playerExplosionSprites);
 
@@ -1359,7 +1425,9 @@ AfternoonStalkerEngine::animateAutomaticCharacters()
     */
     static const char *noEnemies = getenv("NOENEMIES");
 
-    maxNumRobots = 3 + numCreatedRobots / 63;
+    size_t minimum = (rules == AS_RULES_2 ?  5 :  3);
+    size_t step    = (rules == AS_RULES_2 ? 21 : 63);
+    size_t maxNumRobots = minimum + numCreatedRobots / step;
 
     if (timeUntilNextRobot > 0)
 	timeUntilNextRobot--;
@@ -1371,10 +1439,10 @@ AfternoonStalkerEngine::animateAutomaticCharacters()
 
 	const PixmapArray *pa = NULL;
 
-	if (sendOnlyBlueRobots)
-	    pa = &blueRobotPA;
-	else if (numPlayerLives == 0)
+	if (numPlayerLives == 0)
 	    pa = &grayRobotPA;  // only gray robots in demo mode
+	else if (rules == AS_RULES_2)
+	    pa = rand() % 2 ? &blueRobotPA : &grayRobotPA;
 	else
 	{
 	    /*  Set the variables 'smart' and 'pa' according to game level:
@@ -1497,26 +1565,6 @@ AfternoonStalkerEngine::animateAutomaticCharacters()
     }
 
 
-    /*  Show a bomb once in a while
-    */
-    if (!playerHasBomb
-		&& !theBomb.isTicking()
-		&& bombExplosionTicks == 0
-		&& timeUntilNextBomb > 0
-		&& --timeUntilNextBomb == 0)
-    {
-	Couple bombSize = bombPA.getImageSize();
-
-	Couple pos = chooseRandomAllowedTile();
-	pos += (Couple(1, 1) * TILE_SIDE - bombSize) / 2;
-	Sprite *bomb = new Sprite(bombPA,
-				pos, Couple(), Couple(),
-				Couple(2, 2), bombSize - Couple(4, 4));
-	bombSprites.push_back(bomb);
-    }
-
-
-
     /*  Stop blinking for robots when appropriate:
     */
     for (SpriteList::const_iterator itr = robotSprites.begin();
@@ -1529,83 +1577,19 @@ AfternoonStalkerEngine::animateAutomaticCharacters()
 
     /*  Move other sprites:
     */
-    moveEnemyList(robotSprites, GRAY_ROBOT_SPEED_FACTOR);
+    moveEnemyList(robotSprites, rules == AS_RULES_2 ? 4 : 3);
 
     int incr = (numCreatedRobots >= 18);
     moveEnemyList(spiderSprites, SPIDER_SPEED_FACTOR + incr);
     moveEnemyList(batSprites, BAT_SPEED_FACTOR + incr);
 
-    moveBullets(playerBulletSprites, true);
+    moveBullets(playerBulletSprites);
     moveBullets(robotBulletSprites);
 
 
     /*  Make enemies attack the player:
     */
     makeRobotsShoot();
-
-
-    /*  Process the bomb, if applicable:
-    */
-    if (theBomb.tickDown())
-    {
-	playSoundEffect(bombExplodesSound);
-
-	bombExplosionTicks = 20;  // ticks during which explosion is shown
-
-	// Create the bomb explosion sprites:
-	Couple p = theBomb.getPos();
-	int r = theBomb.getExplosionRadius();
-	Couple size = bombExplosionPA.getImageSize();
-
-	for (int level = 0; level <= 2; level++)
-	{
-	    double angleIncr = PI / (2.5 * (1 << level));
-	    double radius = (level + 1) / 3.0 * (r - size.x / 3);
-	    for (double angle = 0.0; angle < TWO_PI; angle += angleIncr)
-	    {
-		double x = p.x + radius * cos(angle);
-		double y = p.y - radius * sin(angle);
-		Couple q = Couple(int(x), int(y));
-		q -= size / 2;
-		Sprite *s = new Sprite(bombExplosionPA, q,
-					Couple(), Couple(), Couple(), Couple());
-		bombExplosionSprites.push_back(s);
-	    }
-	}
-	Sprite *s = new Sprite(bombExplosionPA, p - size / 2,
-				Couple(), Couple(), Couple(), Couple());
-	bombExplosionSprites.push_back(s);
-    }
-
-
-    /*  Process the bomb explosion, if applicable:
-    */
-    if (bombExplosionTicks > 0)
-    {
-	if (--bombExplosionTicks == 0)
-	{
-	    deleteSprites(bombExplosionSprites);
-	    setTimeUntilNextBomb();
-	}
-	else
-	{
-	    // Kill sprites that are inside the explosion radius:
-	    Couple p = theBomb.getPos();
-	    int r = theBomb.getExplosionRadius();
-
-	    killSpritesNearPosition(robotSprites, p, r);
-	    killSpritesNearPosition(robotBulletSprites, p, r);
-	    killSpritesNearPosition(playerBulletSprites, p, r);
-	    killSpritesNearPosition(spiderSprites, p, r);
-	    killSpritesNearPosition(batSprites, p, r);
-
-	    // Kill player if inside the explosion radius:
-	    Couple v = p - playerSprite->getCenterPos();
-	    double dist = hypot(v.x, v.y);
-	    if (dist <= r)
-		makePlayerDie();
-	}
-    }
 }
 
 
@@ -1834,13 +1818,7 @@ AfternoonStalkerEngine::moveEnemyList(SpriteList &slist, int speedFactor)
 
 
 void
-AfternoonStalkerEngine::moveBullets(SpriteList &slist, bool playMissedSound)
-/*
-    If 'playMissedSound' is true, then a "missed" sound will be played
-    if the bullet dies without having hit anything.
-    A bullet is considered to have hit something if its "time to live"
-    field is not zero.
-*/
+AfternoonStalkerEngine::moveBullets(SpriteList &slist)
 {
     for (SpriteList::iterator it = slist.begin(); it != slist.end(); it++)
     {
@@ -1892,8 +1870,6 @@ AfternoonStalkerEngine::moveBullets(SpriteList &slist, bool playMissedSound)
 
 	if (!allowed)
 	{
-	    if (playMissedSound && bullet->getTimeToLive() == 0)
-		playSoundEffect(missedSound);
 	    *it = NULL;
 	    delete bullet;
 	}
@@ -2028,7 +2004,7 @@ AfternoonStalkerEngine::detectCollisions()
     SpriteList::iterator it;
 
 
-    /*  Human vs gun and bomb:
+    /*  Human vs gun:
     */
     if (numPlayerLives > 0)
     {
@@ -2038,26 +2014,16 @@ AfternoonStalkerEngine::detectCollisions()
 	    {
 		playSoundEffect(gunPickupSound);
 
-		numPlayerBullets += BULLETS_PER_GUN;
+		numPlayerBullets += 6;
+
+		if (rules == AS_RULES_2)
+		    numPlayerBullets += numCreatedRobots / 63;
+
 		delete *it;
 		*it = NULL;
 	    }
 	}
 	removeNullElementsFromSpriteList(gunSprites);
-
-
-	for (it = bombSprites.begin(); it != bombSprites.end(); it++)
-	{
-	    if (playerSprite->collidesWithSprite(**it))
-	    {
-		playSoundEffect(gunPickupSound);  // sic
-
-		playerHasBomb = true;
-		delete *it;
-		*it = NULL;
-	    }
-	}
-	removeNullElementsFromSpriteList(bombSprites);
     }
 
 
@@ -2199,8 +2165,6 @@ AfternoonStalkerEngine::detectCollisions()
 	    if (robotBullet->getPixmapArray() == &bigRobotBulletPA
 				&& bullet->collidesWithSprite(*robotBullet))
 	    {
-		playSoundEffect(missedSound);
-
 		delete bullet;
 		bullet = NULL;
 		*itb = NULL;
@@ -2215,8 +2179,6 @@ AfternoonStalkerEngine::detectCollisions()
 
 	if (bullet != NULL && absorbedByBlinking)
 	{
-	    playSoundEffect(missedSound);
-
 	    delete bullet;
 	    bullet = NULL;
 	    *itb = NULL;
@@ -2319,19 +2281,24 @@ AfternoonStalkerEngine::makePlayerDie()
 
     createPlayerExplosion(*playerSprite);
 
-    theBomb.reset();
-
     playerSprite->setPos(initPlayerPos);
     playerSprite->currentPixmapIndex = 0;
     numPlayerBullets = 0;
 
-    addToNumPlayerLives(-1);
+    --numPlayerLives;
 
     if (numPlayerLives == 0)
-    {
-	// The "demo" mode will only show the gray robots.
-	numCreatedRobots = 0;
-    }
+	endCurrentGame();
+}
+
+
+void
+AfternoonStalkerEngine::endCurrentGame()
+{
+    numPlayerLives = 0;
+    numCreatedRobots = 0;
+    currentMenu = &mainMenu;
+    paused = false;
 }
 
 
@@ -2415,7 +2382,7 @@ AfternoonStalkerEngine::createPlayerExplosion(const Sprite &s)
 void
 AfternoonStalkerEngine::addToScore(long n)
 {
-    const long step = 10000;  // add new life at every <step> points
+    const long step = (rules == AS_RULES_2 ? 30000 : 10000);  // add new life at every <step> points
 
     long oldInterval = theScore / step;
 
@@ -2424,7 +2391,7 @@ AfternoonStalkerEngine::addToScore(long n)
     long newInterval = theScore / step;
     if (oldInterval < newInterval)
     {
-	addToNumPlayerLives(1);
+	++numPlayerLives;
 	playSoundEffect(newLifeSound);
     }
 }
@@ -2495,7 +2462,6 @@ AfternoonStalkerEngine::loadPixmaps() throw(PixmapLoadError)
     loadPixmap(big_robot_bullet_1_xpm, bigRobotBulletPA, 1);
     loadPixmap(big_robot_bullet_2_xpm, bigRobotBulletPA, 2);
     loadPixmap(gun_xpm, gunPA, 0);
-    loadPixmap(bomb_xpm, bombPA, 0);
     loadPixmap(human_explosion_0_xpm, playerExplosionPA, 0);
     loadPixmap(gray_robot_0_xpm, grayRobotPA, 0);
     loadPixmap(gray_robot_1_xpm, grayRobotPA, 1);
@@ -2543,10 +2509,6 @@ AfternoonStalkerEngine::loadPixmaps() throw(PixmapLoadError)
     loadPixmap(bat_explosion_0_xpm, batExplosionPA, 0);
 
 
-    loadPixmap(bomb_explosion_0_xpm, bombExplosionPA, 0);
-    loadPixmap(bomb_explosion_1_xpm, bombExplosionPA, 1);
-
-
     /*  Score digits:
     */
     loadPixmap(digit0_xpm, digitPA, 0);
@@ -2669,7 +2631,6 @@ AfternoonStalkerEngine::drawSprites()
     putSpriteList(spiderExplosionSprites);
     putSpriteList(batExplosionSprites);
     putSpriteList(playerExplosionSprites);
-    putSpriteList(bombExplosionSprites);
 
 
     putSpriteList(robotBulletSprites);
@@ -2684,16 +2645,6 @@ AfternoonStalkerEngine::drawSprites()
 	putSpriteList(gunSprites);
 
 
-    // Make the bomb(s) blink if applicable:
-    if (tickCount % FPS < FPS * 3 / 4)
-	putSpriteList(bombSprites);
-
-
-    // Indicate if player has a bomb:
-    if (playerHasBomb)
-	displayMessage(30, "You have a bomb. Press B to lay it, then move away!");
-
-
     // Score sprites:
     for (it = scoreSprites.begin(); it != scoreSprites.end(); it++)
 	putSprite(**it);
@@ -2708,31 +2659,11 @@ AfternoonStalkerEngine::drawSprites()
 void
 AfternoonStalkerEngine::drawScoreBoard()
 {
-    char temp[128];
-    snprintf(temp, sizeof(temp), "Score: %7ld", theScore);
-    writeString(temp, scorePos);
-    snprintf(temp, sizeof(temp), "Lives: %3u\n", numPlayerLives);
-    writeString(temp, playerLivesPos);
-}
-
-
-void
-AfternoonStalkerEngine::displayPauseMessage()
-{
-    displayMessage(0, "PAUSED -- press P to resume");
-}
-
-
-void
-AfternoonStalkerEngine::displayStartMessage()
-{
-    displayMessage(0, "Afternoon Stalker " VERSION " - by Pierre Sarrazin");
-    displayMessage(1, "Move with arrow keys - shoot with Ctrl key");
-    displayMessage(2, "Also shoot with H, J, K and L keys");
-    displayMessage(3, "Press SPACE to start - press Escape to quit");
-
-    if (useSound && theSoundMixer == NULL)
-	displayMessage(4, "*** Sound device not available ***");
+    static const Couple fontdim = getFontDimensions();
+    char temp[256];
+    snprintf(temp, sizeof(temp), "Score: %07ld    Bullets: %-3d  Lives: %-3u",
+				    theScore, numPlayerBullets, numPlayerLives);
+    displayMessage(30, temp);
 }
 
 
@@ -2749,13 +2680,6 @@ AfternoonStalkerEngine::displayMessage(int row, const char *msg)
 }
 
 
-void
-AfternoonStalkerEngine::addToNumPlayerLives(int n)
-{
-    numPlayerLives += n;
-}
-
-
 void
 AfternoonStalkerEngine::playSoundEffect(SoundMixer::Chunk &wb)
 {
