@@ -1,8 +1,8 @@
-/*  $Id: sdlmain.cpp,v 1.11 2004/05/02 04:55:46 sarrazip Exp $
+/*  $Id: sdlmain.cpp,v 1.16 2009/02/27 03:08:33 sarrazip Exp $
     sdlmain.cpp - main() function for Afternoon Stalker
 
     afternoonstalker - A robot-killing video game.
-    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2009 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -45,9 +45,8 @@ static struct option knownOptions[] =
     { "ms-per-frame",  required_argument, NULL, 'm' },
     { "no-sound",      no_argument,       NULL, 'n' },
     { "full-screen",   no_argument,       NULL, 'f' },
-    { "pedantic",      no_argument,       NULL, 'p' },
-    { "only-blue",     no_argument,       NULL, 'b' },
-    { "set-file",      required_argument, NULL, 's' },
+    { "maze",          required_argument, NULL, 's' },
+    { "rules",         required_argument, NULL, 'r' },
 
     { NULL, 0, NULL, 0 }  // marks the end
 };
@@ -81,11 +80,11 @@ displayHelp(ostream &out)
 "--version          Display this program's version number and exit\n"
 "--ms-per-frame=N   N milliseconds per animation frame [default: 55]\n"
 "                   Min. 1, max. 1000.  50 ms means 20 frames per second\n"
-"--no-sound         Disable sound effects [enabled]\n"
 "--full-screen      Attempt to use full screen mode [window mode]\n"
-"--only-blue        Practice mode: only send blue robots\n"
-"--set-file=F       Use file F as the set description file [level1.set]\n"
-"                   F must be in " PKGDATADIR "\n"
+"--maze=M           Select maze M: 1 = original, 2 = alternative\n"
+"--rules=R          Rules to use (R must be 1 or 2, default is 1)\n"
+"                   1 = original Afternoon Stalker rules\n"
+"                   2 = 2008 updated Afternoon Stalker rules\n"
 "\n"
     ;
 }
@@ -97,11 +96,9 @@ int main(int argc, char *argv[])
     /*  Default game parameters:
     */
     int millisecondsPerFrame = 55;
-    bool useSound = true;
-    bool pedantic = false;
     bool fullScreen = false;
-    bool onlyBlueRobots = false;
-    string setFilename = "level1.set";
+    string mazeFilename = "level1.set";
+    AfternoonStalkerEngine::Rules rules = AfternoonStalkerEngine::AS_RULES_1;
 
     #ifdef HAVE_GETOPT_LONG
 
@@ -132,24 +129,40 @@ int main(int argc, char *argv[])
 		}
 		break;
 	    
-	    case 'n':
-		useSound = false;
-		break;
-	    
 	    case 'f':
 		fullScreen = true;
 		break;
 
-	    case 'p':
-		pedantic = true;
-		break;
-
-	    case 'b':
-		onlyBlueRobots = true;
-		break;
-
 	    case 's':
-		setFilename = optarg;
+		{
+		    string m = optarg;
+		    if (m == "1")
+			mazeFilename = "level1.set";
+		    else if (m == "2")
+			mazeFilename = "level2.set";
+		    else
+		    {
+			cerr << "Invalid '--maze' argument \"" << optarg << "\".\n";
+			displayHelp(cerr);
+			return EXIT_FAILURE;
+		    }
+		}
+		break;
+
+	    case 'r':
+		{
+		    string r = optarg;
+		    if (r == "1")
+			rules = AfternoonStalkerEngine::AS_RULES_1;
+		    else if (r == "2")
+			rules = AfternoonStalkerEngine::AS_RULES_2;
+		    else
+		    {
+			cerr << "Invalid '--rules' argument \"" << optarg << "\".\n";
+			displayHelp(cerr);
+			return EXIT_FAILURE;
+		    }
+		}
 		break;
 
 	    case 'v':
@@ -184,8 +197,9 @@ int main(int argc, char *argv[])
     {
 	AfternoonStalkerEngine theAfternoonStalkerEngine(
 			"Afternoon Stalker",
-			useSound, pedantic, fullScreen, onlyBlueRobots,
-			setFilename);
+			fullScreen,
+			mazeFilename,
+			rules);
 	theAfternoonStalkerEngine.run(millisecondsPerFrame);
     }
     catch (const string &s)
